# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from cStringIO import StringIO


def runProfile(portal, profileName):
    setupTool = getToolByName(portal, 'portal_setup')
    setupTool.runAllImportStepsFromProfile(profileName)


def uninstall(portal, reinstall=False):
    out = StringIO()
    if not reinstall:
        runProfile(portal, 'profile-rer.bandi:uninstall')



# -*- coding: utf-8 -*-
from zope.interface import implements
from rer.bandi.interfaces.bando import IBando
from plone.dexterity.content import Container


class Bando(Container):
    implements(IBando)



# -*- coding: utf-8 -*-
from zope.interface import implements
from Acquisition import aq_inner
from plone.app.layout.globals.interfaces import IContextState
from plone.app.layout.globals.context import ContextState as BaseClass

class ContextState(BaseClass):
    """modifica il metodo Folder, per poter aggiungere elementi al documento anche quando è 
       impostato come vista predefinita
    """
    implements(IContextState)

    def folder(self):
        if self.is_structural_folder():
            return aq_inner(self.context)
        else:
            return self.parent()
    




# -*- coding: utf-8 -*-
from plone import api
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.supermodel import model
from rer.bandi import bandiMessageFactory as _
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from plone.directives import form


def getDefaultEnte():
    default_ente = api.portal.get_registry_record(
        'rer.bandi.interfaces.settings.IBandoSettings.default_ente'
    )
    if default_ente:
        return default_ente
    else:
        return None


class IBandoSchema(model.Schema):
    """ A Dexterity schema for Annoucements """

    form.order_after(riferimenti_bando='IRichText.text')
    riferimenti_bando = RichText(
        title=_('riferimenti_bando_label', default=u"References"),
        description=_('riferimenti_bando_help', default=u""),
        required=False
    )

    form.order_after(chiusura_procedimento_bando='IRichText.text')
    chiusura_procedimento_bando = schema.Date(
        title=_(
            'chiusura_procedimento_bando_label',
            default=u"Closing date procedure"
        ),
        description=_('chiusura_procedimento_bando_help', default=u''),
        required=False
    )

    form.order_after(scadenza_bando='IRichText.text')
    scadenza_bando = schema.Datetime(
        title=_('scadenza_bando_label', default=u"Expiration date and time"),
        description=_(
            'scadenza_bando_help',
            default=u"Deadline to participate in the announcement"
        ),
        required=False
    )

    form.order_after(ente_bando='IRichText.text')
    directives.widget('ente_bando', AjaxSelectFieldWidget,
                      vocabulary='rer.bandi.enti.vocabulary')
    ente_bando = schema.Tuple(
        title=_(u'ente_label', default=u'Authority'),
        description=_(u'ente_help', default=u'Select some authorities.'),
        required=False,
        defaultFactory=getDefaultEnte,
        value_type=schema.TextLine(),
        missing_value=None
    )

    form.order_after(destinatari='IRichText.text')
    directives.widget(destinatari=CheckBoxFieldWidget)
    destinatari = schema.List(
        title=_('destinatari_label', default=u"Recipients"),
        description=_('destinatari_help', default=''),
        required=True,
        value_type=schema.Choice(vocabulary='rer.bandi.destinatari.vocabulary')
    )

    form.order_after(tipologia_bando='IRichText.text')
    directives.widget(tipologia_bando=RadioFieldWidget)
    tipologia_bando = schema.Choice(
        title=_('tipologia_bando_label', default=u"Announcement type"),
        description=_('tipologia_bando_help', default=''),
        vocabulary='rer.bandi.tipologia.vocabulary',
        required=True
    )



# -*- coding: utf-8 -*-
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from rer.bandi import bandiMessageFactory as _
from plone.portlet.collection.collection import ICollectionPortlet
from zope import schema
from zope.component import getMultiAdapter, getUtility
from plone.app.portlets.browser import formhelper
from zope.i18n import translate
from plone.app.vocabularies.catalog import CatalogSource
from zope.interface import implements
from plone import api
try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory


class IBandoCollectionPortlet(ICollectionPortlet):
    """A portlet which renders the results of a collection object.
    """

    show_more_text = schema.TextLine(title=_(u"Other text"),
                                     description=_(
                                         u"Alternative text to show in 'other' link."),
                                     required=True,
                                     default=u'Altro\u2026')

    show_more_path = schema.Choice(
                            title=_(u"Internal link"),
                            description=_(u"Insert an internal link. This field override external link field"),
                            required=False,
                            source=CatalogSource())

    show_description = schema.Bool(
        title=u'Mostra descrizione', required=True, default=False)

    show_tipologia_bando = schema.Bool(
        title=u'Mostra tipologia bando', required=True, default=False)

    show_effective = schema.Bool(
        title=u'Mostra data di pubblicazione', required=True, default=False)

    show_scadenza_bando = schema.Bool(
        title=u'Mostra data di scadenza', required=True, default=False)


class Assignment(base.Assignment):

    """
    Portlet assignment.
    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(IBandoCollectionPortlet)

    header = u""
    target_collection = None
    limit = None
    show_more = True

    # parametri da ripulire
    def __init__(self, header=u"", target_collection=None, limit=None, show_more=True, show_more_text=None, show_more_path=None,
                 show_description=False, show_tipologia_bando=False, show_effective=False, show_scadenza_bando=False,
                 uid=None, thumb_scale=None, random=False, show_dates=False, exclude_context=True, no_icons=False, no_thumbs=False):


        # lista di data, che viene passata all'instanza dell'assignment
        self.header = header
        self.target_collection = target_collection
        self.limit = limit
        self.show_more = show_more
        self.show_more_text = show_more_text
        self.show_more_path = show_more_path
        self.show_description = show_description
        self.show_tipologia_bando = show_tipologia_bando
        self.show_effective = show_effective
        self.show_scadenza_bando = show_scadenza_bando

        self.uid = uid
        self.thumb_scale = thumb_scale
        self.random = random
        self.show_dates = show_dates
        self.exclude_context = exclude_context
        self.no_icons = no_icons
        self.no_thumbs = no_thumbs

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen. Here, we use the title that the user gave.
        """
        return self.header


class Renderer(base.Renderer):

    """Portlet renderer.

    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """

    _template = ViewPageTemplateFile('collection.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)
        self.voc_tipologia = getUtility(
            IVocabularyFactory, name='rer.bandi.tipologia.vocabulary')(self.context)

    # Cached version - needs a proper cache key
    # @ram.cache(render_cachekey)
    # def render(self):
    #     if self.available:
    #         return xhtml_compress(self._template())
    #     else:
    #         return ''

    render = _template

    @property
    def available(self):
        #se la lista di risultati è maggiore di zero allora si puo mostrare la portlet
        return len(self.results())

    def collection_url(self):
        collection = self.collection()
        if collection is None:
            return None
        else:
            #torna una lista di url
            return collection.absolute_url()

    def more_target_url(self):
        """
        link target to use (either the default collection or the alternate one)
        """
        if self.data.show_more_path:
            return self.portal().absolute_url() + self.data.show_more_path

        return self.collection_url()

    # controllare bene questa funzione
    def results(self):
        results = []
        resultList = []
        # la collection su cui vogliamo costrire la portlet ?
        collection = self.collection()
        if collection is not None:
            # tornano tutti gli oggetti della mia collezione
            results = collection.queryCatalog()

            resultList = list(results)
            for el in list(results):

                # controllo che gli elementi all interno della lista siano dei bandi
                if not el.ContentTypeClass() == "contenttype-bando":
                    resultList.pop()

            # se è settato un limite e se il limite è maggiore di zero
            if self.data.limit and self.data.limit > 0:
                resultList = resultList[:self.data.limit]

        #ottengo tutti i bandi che devo mostrare nella collection
        return resultList

    def isValidDeadline(self, date):
        """
        """
        if not date:
            return False
        if date.Date() == '2100/12/31':
           #a default date for bandi that don't have a defined deadline
            return False
        return True

    def isTipologiaValid(self, tipologia_bando):
        """
        """
        return tipologia_bando in [x.value for x in self.voc_tipologia._terms]

    @memoize
    def collection(self):
        """ get the collection the portlet is pointing to"""

        # collection_path = self.data.target_collection
        # if not collection_path:
        #     return None

        # if collection_path.startswith('/'):
        #     collection_path = collection_path[1:]

        # if not collection_path:
        #     return None

        # portal = self.portal()
        # return portal.restrictedTraverse(collection_path, default=None)
        # collectionUID = self.data.target_collection
        collectionUID = self.data.uid
        if not collectionUID:
            return ""

        return api.content.get(UID=collectionUID)

    def getScadenzaDate(self, brain):
        date = brain.getScadenza_bando
        long_format = True
        if brain.getScadenza_bando.Time() == '00:00:00':
            # indexer add 1 day to this date, to make a bando ends at midnight
            # of the day-after, if time is not provided
            date = date - 1
            long_format = False
        return api.portal.get_localized_time(
            datetime=date,
            long_format=long_format
        )

    def portal(self):
        portal_state = getMultiAdapter(
            (self.context, self.request), name=u'plone_portal_state')
        return portal_state.portal()

    def getBandoState(self, bando):
        """
        return corretc bando state
        """
        scadenza_bando = bando.getScadenza_bando
        chiusura_procedimento_bando = bando.getChiusura_procedimento_bando
        state = ('open', translate(_(u'Open'), context=self.request))
        if scadenza_bando and scadenza_bando.isPast():
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = (
                    'closed', translate(_(u'Closed'), context=self.request))
            else:
                state = (
                    'inProgress', translate(_(u'In progress'), context=self.request))
        else:
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = (
                    'closed', translate(_(u'Closed'), context=self.request))

        return state

    def has_effective_date(self, bando):
        if bando.EffectiveDate() == 'None':
            return False
        else:
            effective_date = bando.effective.Date()
            return effective_date != 'None' and effective_date != "1969/12/31"


class AddForm(formhelper.AddForm):

    """Portlet add form.

    This is registered in configure.zcml. The form_fields variable tells
    zope.formlib which fields to display. The create() method actually
    constructs the assignment that is being added.
    """

    schema = IBandoCollectionPortlet

    label = _(u"Add Bandi Portlet")
    description = _(
         u"This portlet display a listing of bandi from a Collection.")

    def create(self, data):
        return Assignment(**data)


class EditForm(formhelper.EditForm):

    """Portlet edit form.

    This is registered with configure.zcml. The form_fields variable tells
    zope.formlib which fields to display.
    """

    schema = IBandoCollectionPortlet

    label = _(u"Edit Bandi Portlet")
    description = _(
         u"This portlet display a listing of bandi from a Collection.")



"""Common configuration constants
"""

PROJECTNAME = 'rer.bandi'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'BandoFolderDeepening': 'rer.bandi: Add Bando Folder Deepening',
    'Bando': 'rer.bandi: Add Bando',
}



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from collective.tiles.collection.interfaces import ICollectionTileRenderer
from zope.interface import implementer
from rer.bandi import bandiMessageFactory as _


@implementer(ICollectionTileRenderer)
class View(BrowserView):

    display_name = _('bandi_layout', default='Layout Bandi')



# -*- coding: utf-8 -*-
from rer.bandi.filevocabulary import XMLFileVocabulary
from rer.bandi.psheetvocabulary import PropertySheetVocabulary

from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import queryUtility
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


TipologiaBandoVocabularyFactory = XMLFileVocabulary(
    envvar='PLONE_RER_BANDI_VOCAB',
    vocabulary_name='rer.bandi.tipologia.vocabulary',
    default_terms=[
        ('beni_servizi',
         'Acquisizione beni e servizi'),
        ('agevolazioni',
         'Agevolazioni, finanziamenti, contributi'),
        ('altro', 'Altro'),
    ]
)


class DestinatariVocabularyFactory(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        values = api.portal.get_registry_record(
            'rer.bandi.interfaces.settings.IBandoSettings.default_destinatari')

        l = []
        for i in range(len(values)):
            l.append(tuple(values[i].split('|')))

        terms = [SimpleTerm(value=pair[0], token=pair[0],
                            title=pair[1]) for pair in l]
        return SimpleVocabulary(terms)


DestinatariVocabulary = DestinatariVocabularyFactory()


class EnteVocabularyFactory(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        catalog = api.portal.get_tool('portal_catalog')
        enti = list(catalog._catalog.uniqueValuesFor('getEnte_bando'))

        terms = [SimpleTerm(
            value=ente,
            token=ente,
            title=ente
        )for ente in enti]

        return SimpleVocabulary(terms)


EnteVocabulary = EnteVocabularyFactory()



from zope.interface import implements
from rer.bandi.interfaces.bandofolderdeepening import IBandoFolderDeepening
from plone.dexterity.content import Container
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs


class BandoFolderDeepening(Container):
    implements(IBandoFolderDeepening)



# -*- coding: utf-8 -*-
from plone import api
from Products.Five import BrowserView
from rer.bandi import bandiMessageFactory as _
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implements, Interface
try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory


class ICollectionBandiView(Interface):
    pass


class CollectionBandiView(BrowserView):
    implements(ICollectionBandiView)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.voc_tipologia = getUtility(
            IVocabularyFactory, name='rer.bandi.tipologia.vocabulary')(self.context)

    def getTipologiaTitle(self, key):
        """
        """
        try:
            value = self.voc_tipologia.getTermByToken(key)
            return value.title
        except LookupError:
            return key

    def isValidDeadline(self, date):
        """
        """
        if not date:
            return False
        if date.Date() == '2100/12/31':
            # a default date for bandi that don't have a defined deadline
            return False
        return True

    def getScadenzaDate(self, brain):
        date = brain.getScadenza_bando
        long_format = True
        if brain.getScadenza_bando.Time() == '00:00:00':
            # indexer add 1 day to this date, to make a bando ends at midnight
            # of the day-after, if time is not provided
            date = date -1
            long_format = False
        return api.portal.get_localized_time(
            datetime=date,
            long_format=long_format
        )

    def getBandoState(self, bando):
        """
        return corretc bando state
        """
        scadenza_bando = bando.getScadenza_bando
        chiusura_procedimento_bando = bando.getChiusura_procedimento_bando
        state = ('open', translate(_(u'Open'), context=self.request))
        if scadenza_bando and scadenza_bando.isPast():
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = ('closed', translate(
                    _(u'Closed'), context=self.request))
            else:
                state = ('inProgress', translate(
                    _(u'In progress'), context=self.request))
        else:
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = ('closed', translate(
                    _(u'Closed'), context=self.request))
        return state



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName

from zope.component import getUtility
from plone.registry.interfaces import IRegistry

import logging
logger = logging.getLogger('rer.bandi')


def import_various(context):
    if context.readDataFile('rer.bandi_various.txt') is None:
        return

    portal = context.getSite()
    addKeyToCatalog(portal)
    addDefaultValueToRegistry()


def addKeyToCatalog(portal):
    # inizializzazione degli indici

    setup_tool = portal.portal_setup
    setup_tool.runImportStepFromProfile('profile-rer.bandi:default', 'catalog')
    catalog = getToolByName(portal, 'portal_catalog')
    indexes = catalog.indexes()

    wanted = [
        ('getChiusura_procedimento_bando', 'DateIndex', {
         'indexed_attrs': 'getChiusura_procedimento_bando', }),
        ('getDestinatariBando', 'KeywordIndex', {
            'indexed_attrs': 'getDestinatariBando', }),
        ('getScadenza_bando', 'DateIndex', {
            'indexed_attrs': 'getScadenza_bando', }),
        ('getTipologia_bando', 'FieldIndex', {
            'indexed_attrs': 'getTipologia_bando', }),
        ('getEnte_bando', 'KeywordIndex', {
            'indexed_attrs': 'getEnte_bando', }),
    ]

    indexables = []
    for idx in wanted:
        if idx[0] in indexes:
            logger.info(
                "Found the '%s' index in the catalog, nothing changed.\n" % idx[0])
        else:
            catalog.addIndex(name=idx[0], type=idx[1], extra=idx[2])
            logger.info("Added '%s' (%s) to the catalog.\n" % (idx[0], idx[1]))
            indexables.append(idx[0])
    if len(indexables) > 0:
        logger.info('Indexing new indexes %s.' % ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def addDefaultValueToRegistry():

    DEST = (u'Cittadini|Cittadini',
            u'Imprese|Imprese',
            u'Enti locali|Enti locali',
            u'Associazioni|Associazioni',
            u'Altro|Altro')

    registry = getUtility(IRegistry)
    registry['rer.bandi.interfaces.settings.IBandoSettings.default_destinatari'] = DEST

    enti = (u'Regione Emilia-Romagna', )
    registry['rer.bandi.interfaces.settings.IBandoSettings.default_ente'] = enti



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from plone import api
from rer.bandi import logger
from rer.bandi.setuphandlers import addKeyToCatalog

default_profile = 'profile-rer.bandi:default'


def upgrade(upgrade_product, version):
    """ Decorator for updating the QuickInstaller of a upgrade """
    def wrap_func(fn):
        def wrap_func_args(context, *args):
            p = getToolByName(context, 'portal_quickinstaller').get(
                upgrade_product)
            setattr(p, 'installedversion', version)
            return fn(context, *args)
        return wrap_func_args
    return wrap_func


@upgrade('rer.bandi', '2.1.0')
def to_2(context):
    """
    """
    logger.info('Upgrading rer.bandi to version 2.1.0')
    portal = context.portal_url.getPortalObject()
    addKeyToCatalog(portal)


def migrate_to_2200(context):
    PROFILE_ID = 'profile-rer.bandi:migrate_to_2200'
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(PROFILE_ID)
    setup_tool.runImportStepFromProfile(default_profile, 'catalog')
    logger.info("Reindexing catalog indexes")
    catalog = getToolByName(context, 'portal_catalog')
    bandi = catalog(portal_type="Bando")
    for bando in bandi:
        bando.getObject().reindexObject(idxs=["getChiusura_procedimento_bando",
                                              "getDestinatariBando",
                                              "getScadenza_bando",
                                              "getTipologia_bando"])

    setup_tool.runImportStepFromProfile(
        'profile-rer.bandi:default', 'plone.app.registry')
    setup_tool.runImportStepFromProfile(
        'profile-rer.bandi:default', 'typeinfo')

    logger.info("Migrated to 2.2.0")


def migrate_to_2300(context):
    setup_tool = api.portal.get_tool('portal_setup')
    setup_tool.runImportStepFromProfile(default_profile, 'plone.app.registry')
    logger.info('Add sortable collection criteria')


def migrate_to_2400(context):
    setup_tool = api.portal.get_tool('portal_setup')
    setup_tool.runImportStepFromProfile(default_profile, 'typeinfo')
    logger.info('Upgrading to 2400')


# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.indexer.decorator import indexer
from rer.bandi.interfaces.bando import IBando

# importo il datetime di python
from datetime import datetime

# funzione che riceve un date e torna un datetime con l'ora a zero


def dateToDatetime(d):
    return datetime.combine(d, datetime.min.time())


@indexer(IBando)
def destinatari_bando(object, **kw):
    return getattr(object, 'destinatari', None)


@indexer(IBando)
def getChiusura_procedimento_bando(object, **kw):

    date_chiusura_procedimento_bando = getattr(
        object, 'chiusura_procedimento_bando', None)
    if date_chiusura_procedimento_bando:
        datetime_chiusura_procedimento_bando = dateToDatetime(
            date_chiusura_procedimento_bando)
    else:
        return DateTime('2100/12/31')

    if datetime_chiusura_procedimento_bando:
        return DateTime(datetime_chiusura_procedimento_bando)


@indexer(IBando)
def getScadenza_bando(object, **kw):
    datetime_scadenza_bando = getattr(object, 'scadenza_bando', None)
    if not datetime_scadenza_bando:
        return DateTime('2100/12/31')
    zope_dt_scadenza_bando = DateTime(datetime_scadenza_bando)
    if zope_dt_scadenza_bando.Time() == '00:00:00':
        return zope_dt_scadenza_bando + 1
    else:
        return zope_dt_scadenza_bando


@indexer(IBando)
def getEnte_bando(object, **kw):
    return getattr(object, 'ente_bando', None)


@indexer(IBando)
def getTipologia_bando(object, **kw):
    return getattr(object, 'tipologia_bando', None)


# @indexer(IBando)
# def SearchableTextBandi(obj):
#     pt = getToolByName(api.portal.get(), 'portal_transforms')
#     stream = pt.convertTo('text/plain', obj.text.output, mimetype='text/html')
#
#     text = []
#     li = []
#     li.append(obj.Title())
#     li.append(obj.Description())
#     li.append(stream.getData().strip())
#
#     for string in li:
#         for word in string.split():
#             if word not in text:
#                 if isinstance(word, unicode):
#                     text.append(word.encode('utf-8'))
#                 else:
#                     text.append(word)
#
#     return ' '.join(text)



from zope.interface import Interface
from zope import schema
from plone.app.registry.browser import controlpanel


class IBandoSettings(Interface):
    """
    Settings used for announcements default value
    """

    default_ente = schema.Tuple(
        title=u"default_ente",
        required=False,
        value_type=schema.TextLine(),
        missing_value=None
    )

    default_destinatari = schema.Tuple(
        title=u"default_destinatari_bandi",
        required=False,
        value_type=schema.TextLine(),
        missing_value=None
    )


class BandiSettings(controlpanel.RegistryEditForm):
    schema = IBandoSettings
    id = 'BandiSettings'
    label = u"Impostazioni per i bandi"


class BandiSettingsControlPanel(controlpanel.ControlPanelFormWrapper):

    form = BandiSettings



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from plone.dexterity.browser import add, edit
from plone import api
from rer.bandi.interfaces import IBandoFolderDeepening
from zope.component import getMultiAdapter, getUtility
from zope.interface import implements, Interface
from z3c.form import field

try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory


class AddForm(add.DefaultAddForm):

    def updateWidgets(self):
        add.DefaultAddForm.updateWidgets(self)

        for group in self.groups:
            if group.label == 'Settings':

                manager = field.Fields(group.fields)
                group.fields = manager.select(
                    'IShortName.id',
                    'IAllowDiscussion.allow_discussion',
                    'IExcludeFromNavigation.exclude_from_nav',
                    'ITableOfContents.table_of_contents'
                )


class AddView(add.DefaultAddView):
    form = AddForm


class EditForm(edit.DefaultEditForm):

    def updateWidgets(self):
        edit.DefaultEditForm.updateWidgets(self)

        for group in self.groups:
            if group.label == 'Settings':

                manager = field.Fields(group.fields)
                group.fields = manager.select(
                    'IShortName.id',
                    'IAllowDiscussion.allow_discussion',
                    'IExcludeFromNavigation.exclude_from_nav',
                    'ITableOfContents.table_of_contents'
                )


class EditView(edit.DefaultEditView):
    form = EditForm


class IBandoView(Interface):
    pass


class BandoView(BrowserView):
    implements(IBandoView)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.voc_tipologia = getUtility(
            IVocabularyFactory, name='rer.bandi.tipologia.vocabulary')(self.context)

    def retrieveFolderDeepening(self):
        """Retrieves all Folder Deppening objects contained in Structured Document
        """
        struct_doc = self.context
        values = []
        dfolders = struct_doc.getFolderContents(
            contentFilter={'object_provides': IBandoFolderDeepening.__identifier__})
        for df in dfolders:
            if not df.exclude_from_nav:
                values.append(dict(title=df.Title,
                                   description=df.Description,
                                   url=df.getURL(),
                                   path=df.getPath()
                                   ))
        return values

    def retrieveContentsOfFolderDeepening(self, path_dfolder):
        """Retrieves all objects contained in Folder Deppening
        """

        values = []
        objs = self.context.portal_catalog(
            path={'query': path_dfolder, 'depth': 1},
            sort_on='getObjPositionInParent'
        )
        pp = getToolByName(self.context, 'portal_properties')

        for obj in objs:
            if not obj.getPath() == path_dfolder and not obj.exclude_from_nav:
                dictfields = dict(title=obj.Title,
                                  description=obj.Description,
                                  url=obj.getURL(),
                                  path=obj.getPath(),
                                  )
                if obj.Type == 'Link':
                    dictfields['url'] = obj.getRemoteUrl
                if obj.Type == 'File':
                    dictfields['url'] = obj.getURL() + "/download/file"
                    # obj_file=obj.getObject().getFile()
                    obj_file = obj.getObject().file
                    # if obj_file.meta_type=='ATBlob':
                    #     obj_size=obj_file.get_size()
                    # else:
                    #      obj_size=obj_file.getSize()
                    obj_size = obj_file.size
                    dictfields['filesize'] = self.getSizeString(obj_size)
                else:
                    dictfields['url'] = obj.getURL() + "/view"

                # icon = getMultiAdapter((self.context, self.request, obj), IContentIcon)
                # dictfields['icon'] = icon.html_tag()
                dictfields['type'] = obj.Type
                values.append(dictfields)

        return values

    def getSizeString(self, size):
        const = {'kB': 1024,
                 'MB': 1024 * 1024,
                 'GB': 1024 * 1024 * 1024}
        order = ('GB', 'MB', 'kB')
        smaller = order[-1]
        if not size:
            return '0 %s' % smaller

        if size < const[smaller]:
            return '1 %s' % smaller
        for c in order:
            if size / const[c] > 0:
                break
        return '%.2f %s' % (float(size / float(const[c])), c)

    def getDestinatariNames(self):
        """
        Return the values of destinatari vocabulary
        """
        dest_utility = getUtility(
            IVocabularyFactory, 'rer.bandi.destinatari.vocabulary')
        destinatari = self.context.destinatari
        if not dest_utility:
            return destinatari
        dest_values = []
        dest_vocab = dest_utility(self.context)
        for dest in destinatari:
            try:
                dest_title = dest_vocab.getTerm(dest).title
            except LookupError:
                dest_title = dest
            dest_values.append(dest_title)
        return dest_values

    def getEffectiveDate(self):
        """
        Return effectiveDate
        """
        plone = getMultiAdapter((self.context, self.request), name="plone")
        # da sistemare meglio questa parte
        # restituisce la prima data possibile quando questa non è presente
        time = self.context.effective()

        # controllo che EffectiveDate torni il valore stringa None, se cosi significa che non e stata settata la data di pubblicazione
        # se cosi allora torna None
        if self.context.EffectiveDate() == "None":
            return None
        else:
            return plone.toLocalizedTime(time)

    def getDeadLinePartecipationDate(self):
        """
        Return deadline partecipation date
        """
        date = self.context.scadenza_bando
        long_format = date.strftime('%H:%M:%S') != '00:00:00'
        return api.portal.get_localized_time(
            datetime=date,
            long_format=long_format
        )

    def getAnnouncementCloseDate(self):
        """
        Return Annoucement close date
        """
        plone = getMultiAdapter((self.context, self.request), name="plone")
        time = self.context.chiusura_procedimento_bando
        return time.strftime('%d/%m/%Y')
